package com.fluentadflowreactnativesdk
import com.facebook.react.bridge.*
import com.fluent.adflowsdk.adflow.FluentAdflowStaticWrapper
import com.facebook.react.bridge.ReadableMap
import com.facebook.react.uimanager.UIManagerModule
import com.facebook.react.uimanager.NativeViewHierarchyManager
import android.util.Log

class FluentAdFlowWidgetModule(reactContext: ReactApplicationContext) :
  ReactContextBaseJavaModule(reactContext) {

  init {
  }

  override fun getName(): String {
    return NAME
  }

  @ReactMethod
  fun initSdk(apiKey: String, referrer: String?, onInitializedSucceed: Callback, onInitializedFailure: Callback) {
      FluentAdflowStaticWrapper.init(apiKey, referrer ?: "",
          {
              // Success callback should return an empty array (like iOS)
              onInitializedSucceed.invoke()
          },
          { errorCode, message ->
              // Failure callback should return an array with errorCode and message
              onInitializedFailure.invoke(errorCode, message)
          }
      )
  }

  @ReactMethod
  fun setParams(viewId: Int, params: ReadableMap) {
    Log.d("FluentAdFlowModule", "setParams called with viewId: $viewId and params: $params")

    val uiManager = reactApplicationContext.getNativeModule(UIManagerModule::class.java)
    if (uiManager == null) {
      Log.e("FluentAdFlowModule", "UIManagerModule is null. Unable to proceed.")
      return
    }
    
    uiManager.addUIBlock { nativeViewHierarchyManager ->
        val view = nativeViewHierarchyManager.resolveView(viewId) as? FluentAdFlowWidgetView
      if (view == null) {
        Log.e("FluentAdFlowModule", "No view found for viewId: $viewId")
        return@addUIBlock
      }
        view?.setParams(params)
    }
  }

  companion object {
    const val NAME = "FluentAdFlowSdkManager"
  }
}
