package com.fluentadflowreactnativesdk

import android.content.Context
import android.hardware.display.DisplayManager
import android.util.Log
import android.view.Display
import android.view.View
import android.view.WindowManager
import android.widget.FrameLayout
import com.facebook.react.bridge.ReadableMap
import com.facebook.react.uimanager.ThemedReactContext
import com.fluent.adflowsdk.adflow.FluentAdflowWidget
import com.fluent.adflowsdk.adflow.TelemetrySDK

class FluentAdFlowWidgetView(context: ThemedReactContext) : FrameLayout(context) {
  private var reactContext: ThemedReactContext? = context
  private lateinit var adflowWidget: FluentAdflowWidget
  private var displayManager: DisplayManager
  private val displayListener: DisplayManager.DisplayListener

  init {
    adflowWidget = FluentAdflowWidget().init(this.reactContext)
    val sampleAdComponent: View = adflowWidget.getView()

    // Set layout params to match parent for both FluentAdFlowWidgetView and its child
    layoutParams = LayoutParams(LayoutParams.MATCH_PARENT, LayoutParams.MATCH_PARENT)
    sampleAdComponent.layoutParams = LayoutParams(LayoutParams.MATCH_PARENT, LayoutParams.MATCH_PARENT)

    addView(sampleAdComponent)

    // Initialize DisplayManager
    displayManager = context.getSystemService(Context.DISPLAY_SERVICE) as DisplayManager

    // Initialize DisplayListener
    displayListener = object : DisplayManager.DisplayListener {
      override fun onDisplayChanged(displayId: Int) {
        if (displayId == Display.DEFAULT_DISPLAY) {
          handleScreenRotation()
        }
      }

      override fun onDisplayAdded(displayId: Int) {}
      override fun onDisplayRemoved(displayId: Int) {}
    }

    // Register the display listener
    displayManager.registerDisplayListener(displayListener, null)
  }

  public fun setParams(params: ReadableMap) {
    Log.d("FluentAdFlowWidgetView", "Setting params: $params")
    val paramsMap = convertReadableMapToHashMap(params)

    // Remove the existing adFlowListener if present
    if (adflowWidget.adFlowListener != null) {
      adflowWidget.setAdFlowListener(null) // Clear the existing listener
    }

    // Set the new adFlowListener
    adflowWidget.setAdFlowListener(AdFlowListener(reactContext, paramsMap.getOrElse("viewId") { "" }))
    adflowWidget.setParams(paramsMap)
  }

  private fun handleScreenRotation() {
    val rotation = (context.getSystemService(Context.WINDOW_SERVICE) as WindowManager)
      .defaultDisplay.rotation

    when (rotation) {
      0 -> Log.d("FluentAdFlowWidgetView", "Rotation: Portrait (0°)")
      1 -> Log.d("FluentAdFlowWidgetView", "Rotation: Landscape (90°)")
      2 -> Log.d("FluentAdFlowWidgetView", "Rotation: Upside Down (180°)")
      3 -> Log.d("FluentAdFlowWidgetView", "Rotation: Reverse Landscape (270°)")
    }

    // Resize the view dynamically
    adflowWidget.getView().layoutParams = LayoutParams(LayoutParams.MATCH_PARENT, LayoutParams.MATCH_PARENT)
    adflowWidget.getView().requestLayout()

    layoutParams = LayoutParams(LayoutParams.MATCH_PARENT, LayoutParams.MATCH_PARENT)
    requestLayout()
  }

  fun convertReadableMapToHashMap(readableMap: ReadableMap): HashMap<String, String> {
    val hashMap = HashMap<String, String>()
    val iterator = readableMap.keySetIterator()

    while (iterator.hasNextKey()) {
      val key = iterator.nextKey()
      val value = readableMap.getString(key)
      if (value != null) {
        hashMap[key] = value
      }
    }

    return hashMap
  }

  public fun setScrollingEnabled(isScrollingEnabled: Boolean) {
    adflowWidget.setScrollingEnabled(isScrollingEnabled)
  }

  public fun setSdkWrapperName(wrapperName: String) {
    TelemetrySDK.sdkWrapperName = wrapperName
  }

  public fun setSdkWrapperVersion(wrapperVersion: String) {
    TelemetrySDK.sdkWrapperVersion = wrapperVersion
  }

  override fun onDetachedFromWindow() {
    super.onDetachedFromWindow()
    displayManager.unregisterDisplayListener(displayListener)
  }
}
