import Foundation
import React
import FluentAdFlowAdsWidget

@objc(FluentAdFlowSdkManager)
class FluentAdFlowSdkManager: NSObject, RCTBridgeModule {
    static func moduleName() -> String {
        return "RNFluentAdFlowAdsSdk"
    }

    private let logTag = "FluentAdFlowSdkManager"

    @objc(initSdk:withReferrer:onInitializedSucceed:onInitializedFailure:)
    func initSdk(_ apiKey: String, 
                 withReferrer referrer: String?, 
                 onInitializedSucceed: @escaping RCTResponseSenderBlock, 
                 onInitializedFailure: @escaping RCTResponseSenderBlock) {
        
        FluentAdFlowAdsSdk().initSdk(apiKey, withReferrer: referrer, 
            onInitializedSucceed: { 
                // Success callback should return an empty array
                onInitializedSucceed([])
            }, 
            onInitializedFailure: { (errorCode, message) in
                // Failure callback should return an array with errorCode and message
                onInitializedFailure([errorCode, message])
            }
        )
    }

    @objc func setParams(_ viewId: NSNumber, params: NSDictionary) {
        print("Bridge - viewId: \(viewId)")

        // Ensure execution on the UI thread
        DispatchQueue.main.async {
            guard
                let bridge = RCTBridge.current(),
                let uiManager = bridge.module(for: RCTUIManager.self) as? RCTUIManager
            else {
                return
            }
            
            // Access the view for the given viewId on the main thread
            if let fluentAdFlowWidgetView = uiManager.view(forReactTag: viewId) as? FluentAdFlowWidgetView {
                fluentAdFlowWidgetView.setParams(params)
            }
        }
    }

    @objc
    static func requiresMainQueueSetup() -> Bool {
        return true // Change this to true if UI-related work is being done
    }
}
