import React
import FluentAdFlowAdsWidget

@available(iOS 14, *)
@objc(FluentAdFlowWidgetViewManager)
class FluentAdFlowWidgetViewManager: RCTViewManager {
    private var widgets: [String: FluentAdFlowWidget] = [:] // Map of viewId to FluentAdFlowWidget
    private var nextViewId: Int = 1 // Incremental ID generator

    override func view() -> (UIView) {
        let widget = FluentAdFlowWidget()
        let viewId = "view_\(nextViewId)"
        nextViewId += 1

        widget.viewId = viewId // Set viewId to the widget if it supports it

        // Add listeners for events
        widget.events.listenTo(eventName: "onAdFlowContainerClosed") { [weak self] data, viewId in
            self?.onAdFlowContainerClosed(data: data, viewId: viewId)
        }
        widget.events.listenTo(eventName: "onAdFlowContainerHeightChanged") { [weak self] data, viewId in
            self?.onAdFlowContainerHeightChanged(data: data, viewId: viewId)
        }
        widget.events.listenTo(eventName: "onAdFlowContainerShow") { [weak self] data, viewId in
            self?.onAdFlowContainerShow(data: data, viewId: viewId)
        }
        widget.events.listenTo(eventName: "onValidationFailed") { [weak self] data, viewId in
            self?.onValidationFailed(data: data, viewId: viewId)
        }
        widget.events.listenTo(eventName: "onError") { [weak self] data, viewId in
            self?.onError(data: data, viewId: viewId)
        }
        widget.events.listenTo(eventName: "onNotified") { [weak self] data, viewId in
            self?.onNotified(data: data, viewId: viewId)
        }
        widget.events.listenTo(eventName: "onInternalAdShown") { [weak self] data, viewId in
            self?.onInternalAdShown(data: data, viewId: viewId)
        }
        widget.events.listenTo(eventName: "onInternalAdClosed") { [weak self] data, viewId in
            self?.onInternalAdClosed(data: data, viewId: viewId)
        }
      
        return widget
    }

    deinit {
        widgets.values.forEach { $0.events.removeListeners(eventNameToRemoveOrNil: nil) }
        widgets.removeAll()
    }

    @objc override static func requiresMainQueueSetup() -> Bool {
        return false
    }

    // Event Handlers
    func onAdFlowContainerClosed(data: Any?, viewId: String?) {
        AdsViewEventEmitter.emitter.sendEvent(withName: "onAdFlowContainerClosed", body: [viewId])
    }

    func onValidationFailed(data: Any?, viewId: String?) {
        AdsViewEventEmitter.emitter.sendEvent(withName: "onValidationFailed", body: [viewId])
    }

    func onError(data: Any?, viewId: String?) {
        AdsViewEventEmitter.emitter.sendEvent(withName: "onError", body: [viewId])
    }

    func onAdFlowContainerShow(data: Any?, viewId: String?) {
        AdsViewEventEmitter.emitter.sendEvent(withName: "onAdFlowContainerShow", body: [viewId])
    }

    func onAdFlowContainerHeightChanged(data: Any?, viewId: String?) {
        let result = (viewId ?? "") + "$\(data as? CGFloat ?? 0.0)"
        AdsViewEventEmitter.emitter.sendEvent(withName: "onAdFlowContainerHeightChanged", body: result)
    }

    func onNotified(data: Any?, viewId: String?) {
        AdsViewEventEmitter.emitter.sendEvent(withName: "onNotified", body: [viewId])
    }

    func onInternalAdShown(data: Any?, viewId: String?) {
        AdsViewEventEmitter.emitter.sendEvent(withName: "onInternalAdShown", body: [viewId])
    }

    func onInternalAdClosed(data: Any?, viewId: String?) {
        AdsViewEventEmitter.emitter.sendEvent(withName: "onInternalAdClosed", body: [viewId])
    }
}

@available(iOS 14, *)
class FluentAdFlowWidget: FluentAdFlowWidgetView {
    var _currentProps: [String]! = nil

    required init?(apiKey: String, referrer: String) {
        super.init(apiKey: apiKey, referrer: referrer)
    }

    required init(frame: CGRect) {
        super.init(frame: frame)
    }

    required init(frame: CGRect, apiClient: ApiClientProtocol) {
        fatalError("init(frame:apiClient:) has not been implemented")
    }

    required init?(coder aDecoder: NSCoder) {
        fatalError("init(coder:) has not been implemented")
    }

    @MainActor required init?(apiKey: String, referrer: String?) {
        fatalError("init(apiKey:referrer:) has not been implemented")
    }

    @MainActor required init(frame: CGRect, viewId: String) {
        fatalError("init(frame:viewId:) has not been implemented")
    }

    override func setParams(_ params: NSDictionary) {
        super.setParams(params)
    }
}

@objc(AdsViewEventEmitter)
@available(iOS 14, *)
open class AdsViewEventEmitter: RCTEventEmitter {
    public static var emitter: AdsViewEventEmitter!

    override init() {
        super.init()
        AdsViewEventEmitter.emitter = self
    }

    @objc
    override public func supportedEvents() -> [String] {
        return [
            "onReady",
            "onPending",
            "onError",
            "onValidationFailed",
            "onAdFlowContainerClosed",
            "onAdFlowContainerShow",
            "onAdFlowContainerHeightChanged",
            "onInternalAdClosed",
            "onInternalAdShown",
            "onNotified"
        ]
    }

    @objc override public static func requiresMainQueueSetup() -> Bool {
        return false
    }
}
