import { NativeModules } from 'react-native';
import { FluentAdFlowWidgetView } from './fluent-adflow-widget';
import DebugLogger from './DebugLogger';

// Safely access FluentAdFlowSdkManager
const FluentAdFlowSdkManager = NativeModules.FluentAdFlowSdkManager || {
  initSdk: () => {
    DebugLogger.logMessage('The SDK Manager is not available.');
    return Promise.reject({
      errorCode: 4001,
      message: 'The SDK Manager is not available.'
    });
  }
};
export class FluentAdFlowWidget {
  /**
   * Initialize the FluentAdFlow SDK.
   * @param {string} apiKey - API key for initialization.
   * @param {string} [referrer] - Optional referrer value.
   * @returns {Promise<void>}
   */
  static init(apiKey, referrer) {
    return new Promise((resolve, reject) => {
      // Use the version with success & failure callbacks
      FluentAdFlowSdkManager.initSdk(apiKey, referrer ?? '', () => resolve(),
      // Success callback
      (errorCode, message) => reject({
        errorCode,
        message
      }) // Reject with raw values
      );
    });
  }
}

export { FluentAdFlowWidgetView };
//# sourceMappingURL=index.js.map